@@ -3,8 +3,6 @@ from ..attributes import Animation, Data, Color
 from ..utils import *
 import uuid
 
-DEFAULT_COLOR = Color(128,128,128)
-
 class Object(BaseObject):
     """
     Object class. Defines a generic object in the ARENA.
@@ -66,9 +64,6 @@ class Object(BaseObject):
         self.update_handler = update_handler
         self.animations = []
 
-        # add default color
-        self.update_attributes(color=DEFAULT_COLOR)
-
         # add current object to all_objects dict
         Object.add(self)
 
@@ -164,6 +159,9 @@ class Object(BaseObject):
             elif "goto_url" == k:
                 json_data["goto-url"] = v
 
+            elif "jitsi_video" == k:
+                json_data["jitsi-video"] = v
+
             elif "collision_listener" == k:
                 json_data["collision-listener"] = v
 
