@@ -1,30 +1,31 @@
-import os
-import sys
 import json
-import random
-import socket
-import asyncio
-
-import paho.mqtt.client as mqtt
+import os
+import re
+import sys
 from datetime import datetime
 from inspect import signature
 
+from .arena_mqtt import ArenaMQTT
 from .attributes import *
-from .objects import *
 from .events import *
+from .objects import *
 from .utils import *
-from .event_loop import *
 
-from . import auth
 
-class Scene(object):
+class Scene(ArenaMQTT):
     """
     Gives access to an ARENA scene.
-    Wrapper around Paho MQTT client and EventLoop.
     Can create and execute various user-defined functions/tasks.
+
+    :param str host: Hostname of the MQTT broker (required).
+    :param str realm: Reserved topic fork for future use (optional).
+    :param str namespace: Username of authenticated user or other namespace (automatic).
+    :param str scene: The name of the scene, without namespace (required).
     """
+
     def __init__(
                 self,
+                realm = "realm",
                 network_latency_interval = 10000,  # run network latency update every 10s
                 on_msg_callback = None,
                 new_obj_callback = None,
@@ -32,235 +33,66 @@ class Scene(object):
                 user_left_callback = None,
                 delete_obj_callback = None,
                 end_program_callback = None,
+                video = False,
                 debug = False,
+                cli_args = False,
                 **kwargs
             ):
-        if os.environ.get("MQTTH"):
-            self.host  = os.environ["MQTTH"]
-        elif "host" in kwargs and kwargs["host"] is not None:
-            self.host = kwargs["host"]
-            print("Cannot find MQTTH environmental variable, using input parameter instead.")
-        else:
-            sys.exit("mqtt host argument (host) is unspecified or None, aborting...")
-
-        if os.environ.get("REALM"):
-            self.realm  = os.environ["REALM"]
-        elif "realm" in kwargs and kwargs["realm"] is not None:
-            self.realm = kwargs["realm"]
-            print("Cannot find REALM environmental variable, using input parameter instead.")
-        else:
-            sys.exit("realm argument (realm) is unspecified or None, aborting...")
+        if cli_args:
+            self.args = self.parse_cli()
+            if self.args["mqtth"]:
+                kwargs["host"] = self.args["mqtth"]
+            if self.args["namespace"]:
+                kwargs["namespace"] = self.args["namespace"]
+            if self.args["scene"]:
+                kwargs["scene"] = self.args["scene"]
 
         if os.environ.get("SCENE"):
-            self.scene  = os.environ["SCENE"]
-        elif "scene" in kwargs and kwargs["scene"] is not None:
+            self.scene = os.environ["SCENE"]
+        elif "scene" in kwargs and kwargs["scene"]:
+            if re.search("/", kwargs["scene"]):
+                sys.exit("scene argument (scene) cannot include '/', aborting...")
             self.scene = kwargs["scene"]
             print("Cannot find SCENE environmental variable, using input parameter instead.")
         else:
             sys.exit("scene argument (scene) is unspecified or None, aborting...")
-
-        self.debug = debug
-
-        print("=====")
-        # do user auth
-        username = None
-        password = None
-        if os.environ.get("ARENA_USERNAME") and os.environ.get("ARENA_PASSWORD"):
-            # auth 1st: use passed in env var
-            username = os.environ["ARENA_USERNAME"]
-            password = os.environ["ARENA_PASSWORD"]
-            auth.store_environment_auth(username, password)
-        else:
-            local = auth.check_local_auth()
-            if local and 'username' in local and 'token' in local:
-                # auth 2nd: use locally saved token
-                username = local["username"]
-                password = local["token"]
-            else:
-                # auth 3rd: use the user account online
-                username = auth.authenticate_user(self.host)
-
-        if os.environ.get("NAMESPACE"):
-            self.namespace = os.environ["NAMESPACE"]
-        elif "namespace" not in kwargs or ("namespace" in kwargs and kwargs["namespace"] is None):
-            self.namespace = username
-        else:
-            self.namespace = kwargs["namespace"]
-
-        self.mqttc_id = "pyClient-" + self.generate_client_id()
-
-        # set up scene variables
-        self.namespaced_scene =  f"{self.namespace}/{self.scene}"
-
-        self.root_topic = f"{self.realm}/s/{self.namespaced_scene}"
-        self.scene_topic = f"{self.root_topic}/#"   # main topic for entire scene
-        self.persist_url = f"https://{self.host}/persist/{self.namespaced_scene}"
-
-        self.latency_topic = "$NETWORK/latency"     # network graph latency update
-        self.ignore_topic = f"{self.root_topic}/{self.mqttc_id}/#" # ignore own messages
-
-        self.mqttc = mqtt.Client(
-            self.mqttc_id, clean_session=True
+        super().__init__(
+            realm,
+            network_latency_interval,
+            on_msg_callback,
+            end_program_callback,
+            video,
+            debug,
+            **kwargs
         )
 
-        # do scene auth
-        if username is None or password is None:
-            data = auth.authenticate_scene(
-                            self.host, self.realm,
-                            self.namespaced_scene, username
-                        )
-            if 'username' in data and 'token' in data:
-                username = data["username"]
-                password = data["token"]
-        self.mqttc.username_pw_set(username=username, password=password)
-        print("=====")
+        self.jitsi_host = self.config_data["ARENADefaults"]["jitsiHost"]
+        self.persist_host = self.config_data["ARENADefaults"]["persistHost"]
+        self.persist_path = self.config_data["ARENADefaults"]["persistPath"]
+
+        self.persist_url = f"https://{self.persist_host}{self.persist_path}{self.namespaced_target}"
 
         # set up callbacks
-        self.on_msg_callback = on_msg_callback
         self.new_obj_callback = new_obj_callback
         self.delete_obj_callback = delete_obj_callback
         self.user_join_callback = user_join_callback
         self.user_left_callback = user_left_callback
-        self.end_program_callback = end_program_callback
 
         self.unspecified_object_ids = set() # objects that exist in the scene,
                                           # but this scene instance does not
                                           # have a reference to
         self.users = {} # dict of all users
 
-        self.event_loop = EventLoop(self.disconnect)
-
-        aioh = AsyncioMQTTHelper(self.event_loop, self.mqttc)
-
-        # have all tasks wait until mqtt client is connected before starting
-        self.mqtt_connect_evt = asyncio.Event()
-        self.mqtt_connect_evt.clear()
-
-        # set paho mqtt callbacks
-        self.mqttc.on_connect = self.on_connect
-        self.mqttc.on_disconnect = self.on_disconnect
-
-        # add main message processing + callbacks loop to tasks
-        self.run_async(self.process_message)
-
-        # update network latency every network_latency_interval secs
-        self.run_forever(self.network_latency_update,
-                         interval_ms=network_latency_interval)
-
-        self.msg_queue = asyncio.Queue()
-
-        # connect to mqtt broker
-        if "port" in kwargs:
-            self.mqttc.connect(self.host, kwargs["port"])
-        else:
-            self.mqttc.connect(self.host)
-        self.mqttc.socket().setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 2048)
-
         print(f"Loading: https://{self.host}/{self.namespace}/{self.scene}, realm={self.realm}")
 
-    def generate_client_id(self):
-        """Returns a random 6 digit id"""
-        return str(random.randrange(100000, 999999))
-
-    def network_latency_update(self):
-        """Update client latency in $NETWORK/latency"""
-        # publish empty message with QoS of 2 to update latency
-        self.mqttc.publish(self.latency_topic, "", qos=2)
-
-    def run_once(self, func=None, **kwargs):
-        """Runs a user-defined function on startup"""
-        if func is not None:
-            w = SingleWorker(self.event_loop, func, self.mqtt_connect_evt, **kwargs)
-            self.event_loop.add_task(w)
-        else:
-            # if there is no func, we are in a decorator
-            def _run_once(func):
-                self.run_once(func, **kwargs)
-                return func
-            return _run_once
-
-    def run_after_interval(self, func=None, interval_ms=1000, **kwargs):
-        """Runs a user-defined function after a interval_ms milliseconds"""
-        if func is not None:
-            if interval_ms < 0:
-                print("Invalid interval! Defaulting to 1000ms")
-                interval_ms = 1000
-            w = LazyWorker(self.event_loop, func, self.mqtt_connect_evt, interval_ms, **kwargs)
-            self.event_loop.add_task(w)
-        else:
-            # if there is no func, we are in a decorator
-            def _run_after_interval(func):
-                self.run_after_interval(func, interval_ms, **kwargs)
-                return func
-            return _run_after_interval
-
-    def run_async(self, func=None, **kwargs):
-        """Runs a user defined aynscio function"""
-        if func is not None:
-            w = AsyncWorker(self.event_loop, func, self.mqtt_connect_evt, **kwargs)
-            self.event_loop.add_task(w)
-        else:
-            # if there is no func, we are in a decorator
-            def _run_async(func):
-                self.run_async(func, **kwargs)
-                return func
-            return _run_async
-
-    def run_forever(self, func=None, interval_ms=1000, **kwargs):
-        """Runs a function every interval_ms milliseconds"""
-        if func is not None:
-            if interval_ms < 0:
-                print("Invalid interval! Defaulting to 1000ms")
-                interval_ms = 1000
-            t = PersistentWorker(self.event_loop, func, self.mqtt_connect_evt, interval_ms, **kwargs)
-            self.event_loop.add_task(t)
-        else:
-            # if there is no func, we are in a decorator
-            def _run_forever(func):
-                self.run_forever(func, interval_ms, **kwargs)
-                return func
-            return _run_forever
-
-    def run_tasks(self):
-        """Run event loop"""
-        print("Connecting to the ARENA... ", end="")
-        self.event_loop.run()
-
-    def stop_tasks(self):
-        """Stop event loop"""
-        self.event_loop.stop()
-
-    async def sleep(self, interval_ms):
-        """Public function for sleeping in async functions"""
-        await asyncio.sleep(interval_ms / 1000)
-
     def on_connect(self, client, userdata, flags, rc):
-        """Paho MQTT client on_connect callback"""
+        super().on_connect(client, userdata, flags, rc)
         if rc == 0:
-            self.mqtt_connect_evt.set()
-
-            # listen to all messages in scene
-            client.subscribe(self.scene_topic)
-            client.message_callback_add(self.scene_topic, self.on_message)
-
             # create ARENA-py Objects from persist server
             # no need to return anything here
             self.get_persisted_objs()
 
-            print("Connected!")
-            print("=====")
-        else:
-            print(f"Connection error! Result code: {rc}")
-
-    def on_message(self, client, userdata, msg):
-        # ignore own messages
-        if mqtt.topic_matches_sub(self.ignore_topic, msg.topic):
-            return
-
-        self.msg_queue.put_nowait(msg)
-
     async def process_message(self):
-        """Main message processing function"""
         while True:
             msg = await self.msg_queue.get()
 
@@ -310,6 +142,7 @@ class Scene(object):
                                         )
                             elif self.delete_obj_callback:
                                 self.callback_wrapper(self.delete_obj_callback, obj, payload)
+                            Object.remove(obj)
                             continue
 
                         else: # create/update
@@ -354,19 +187,6 @@ class Scene(object):
         else:
             func(self, arg, msg)
 
-    def on_disconnect(self, client, userdata, rc):
-        """Paho MQTT client on_disconnect callback"""
-        if rc == 0:
-            print("Disconnected from the ARENA!")
-        else:
-            print(f"Disconnect error! Result code: {rc}")
-
-    def disconnect(self):
-        """Disconnects Paho MQTT client"""
-        if self.end_program_callback:
-            self.end_program_callback(self)
-        self.mqttc.disconnect()
-
     def generate_custom_event(self, evt, action="clientEvent"):
         """Publishes an custom event. Could be user or library defined"""
         return self._publish(evt, action)
@@ -515,7 +335,7 @@ class Scene(object):
             obj.persist = True
         else:
             # pass token to persist
-            data = auth.urlopen(url=f"{self.persist_url}/{object_id}", creds=True)
+            data = self.auth.urlopen(url=f"{self.persist_url}/{object_id}", creds=True)
             output = json.loads(data)
             if len(output) > 0:
                 output = output[0]
@@ -534,7 +354,7 @@ class Scene(object):
         """Returns a dictionary of persisted objects. [TODO] check object_type"""
         objs = {}
         # pass token to persist
-        data = auth.urlopen(url=self.persist_url, creds=True)
+        data = self.auth.urlopen(url=self.persist_url, creds=True)
         output = json.loads(data)
         for obj in output:
             if obj["type"] == Object.object_type or obj["type"] == Object.type:
@@ -558,7 +378,7 @@ class Scene(object):
         """Returns a dictionary for scene-options. [TODO] wrap the output as a BaseObject"""
         scene_opts_url = f"{self.persist_url}?type=scene-options"
         # pass token to persist
-        data = auth.urlopen(url=scene_opts_url, creds=True )
+        data = self.auth.urlopen(url=scene_opts_url, creds=True )
         output = json.loads(data)
         return output
 
@@ -566,17 +386,7 @@ class Scene(object):
         """ Request list of scene names for logged in user account that user has publish permission for.
         Returns: list of scenes.
         """
-        return auth.get_writable_scenes(host=self.host)
-
-    def message_callback_add(self, sub, callback):
-        """Subscribes to new topic and adds callback"""
-        self.mqttc.subscribe(sub)
-        self.mqttc.message_callback_add(sub, callback)
-
-    def message_callback_remove(self, sub):
-        """Unsubscribes to topic and removes callback"""
-        self.mqttc.unsubscribe(sub)
-        self.mqttc.message_callback_remove(sub)
+        return self.auth.get_writable_scenes(host=self.host)
 
     def get_user_list(self):
         """Returns a list of users"""
